#!/bin/sh
#
#	Define two separate executables to be compiled from single
#	.c files via separate Conscript files in separate
#	subdirectories.  Prune the build to one of the subdirectories
#	('+subdir') and make sure only the executable in that
#	subdirectory gets built.  Invoke cons with '.' as an
#	argument.  Make sure the other subdirectory's executable
#	got built, too.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [build pruning]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	if test X$PRESERVE = X -a X$1 = X ; then
		cd $here
		find $work -type d -user $USER -exec chmod u+w {} \;
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

mkdir foo
if test $? -ne 0 ; then no_result ; fi
mkdir bar
if test $? -ne 0 ; then no_result ; fi

#
cat <<'_EOF_' >Construct
Build qw (
	foo/Conscript
	bar/Conscript
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >foo/Conscript
$env = new cons;
Program $env 'foo', 'foo.c';
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >foo/foo.c
main()
{
	printf("This is the foo/foo.c file.\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >bar/Conscript
$env = new cons;
Program $env 'bar', 'bar.c';
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >bar/bar.c
main()
{
	printf("This is the bar/bar.c file.\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

$prog . +bar > cons.1.out 2>&1
if test $? -ne 0 ; then fail ; fi
if test -x foo/foo ; then fail ; fi

bar/bar > bar.out
if test $? -ne 0 ; then fail ; fi
diff - bar.out <<'_EOF_'
This is the bar/bar.c file.
_EOF_
if test $? -ne 0 ; then fail ; fi

$prog . > cons.2.out 2>&1
if test $? -ne 0 ; then fail ; fi

foo/foo > foo.out
if test $? -ne 0 ; then fail ; fi
diff - foo.out <<'_EOF_'
This is the foo/foo.c file.
_EOF_
if test $? -ne 0 ; then fail ; fi

#
pass
