#!/bin/sh
#
#	Create a subdirectory with three .c files and construct
#	file.  Create decoy .c files in the current directory.
#	Invoke cons -f subdir/Construct.  Check that it built
#	correctly in the subdirectory and didn't pick up any decoy
#	of the .c files.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [-f dir/Construct]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	if test X$PRESERVE = X -a X$1 = X ; then
		cd $here
		find $work -type d -user $USER -exec chmod u+w {} \;
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

mkdir subdir
if test $? -ne 0 ; then exit 2 ; fi

#
cat <<'_EOF_' >subdir/Construct
$env = new cons;
Program $env 'foo', qw (
	a.c
	b.c
	main.c
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >a.c
a()
{
	printf("a.c\n");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >b.c
b()
{
	printf("b.c\n");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >subdir/a.c
a()
{
	printf("subdir/a.c\n");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >subdir/b.c
b()
{
	printf("subdir/b.c\n");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >subdir/main.c
main()
{
	a();
	b();
	printf("subdir/main.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

$prog -f subdir/Construct . >cons.1.out 2>cons.1.err
if test $? -ne 0 ; then fail ; fi
subdir/foo > foo.out
if test $? -ne 0 ; then fail ; fi
diff - foo.out <<'_EOF_'
subdir/a.c
subdir/b.c
subdir/main.c
_EOF_
if test $? -ne 0 ; then fail ; fi

$prog -f subdir/Construct . >cons.2.out 2>cons.2.err
if test $? -ne 0 ; then fail 2 cons.2.out ; fi
diff - cons.2.out <<'_EOF_'
cons: "." is up-to-date.
_EOF_
if test $? -ne 0 ; then fail 2 ; fi
diff - cons.2.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 2 ; fi

#
pass
