#!/bin/sh
#
#	Create Construct and Conscript files for creating four
#	executables, two from .c files in the local directory and
#	two from .c files in a subdirectory.  Invoke cons -d.
#	Compare the output against expected results.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [-d]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	if test X$PRESERVE = X -a X$1 = X ; then
		cd $here
		find $work -type d -user $USER -exec chmod u+w {} \;
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

mkdir subdir
if test $? -ne 0 ; then no_result ; fi

#
cat <<'_EOF_' >Construct
$env = new cons;
Program $env 'a', 'a.c';
Program $env 'b', 'b.c';
Build 'subdir/Conscript';
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >a.c
main()
{
	printf("a.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >b.c
main()
{
	printf("b.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >subdir/Conscript
$env = new cons;
Program $env 'c', 'c.c';
Program $env 'd', 'd.c';
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >subdir/c.c
main()
{
	printf("subdir/c.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >subdir/d.c
main()
{
	printf("subdir/d.c\n");
	exit (0);
}
_EOF_

$prog -d . >cons.1.out 2>cons.1.err
if test $? -ne 0 ; then fail 1 ; fi
diff - cons.1.out <<'_EOF_'
a
  a.o
    a.c
cc -c a.c -o a.o
cc -o a a.o
b
  b.o
    b.c
cc -c b.c -o b.o
cc -o b b.o
subdir/c
  subdir/c.o
    subdir/c.c
cc -c subdir/c.c -o subdir/c.o
cc -o subdir/c subdir/c.o
subdir/d
  subdir/d.o
    subdir/d.c
cc -c subdir/d.c -o subdir/d.o
cc -o subdir/d subdir/d.o
_EOF_
if test $? -ne 0 ; then fail 1 ; fi
./a >a.1.out
if test $? -ne 0 ; then fail 1 ; fi
diff - a.1.out <<'_EOF_'
a.c
_EOF_
if test $? -ne 0 ; then fail 1 ; fi
./b >b.1.out
if test $? -ne 0 ; then fail 1 ; fi
diff - b.1.out <<'_EOF_'
b.c
_EOF_
if test $? -ne 0 ; then fail 1 ; fi
subdir/c >c.1.out
if test $? -ne 0 ; then fail 1 ; fi
diff - c.1.out <<'_EOF_'
subdir/c.c
_EOF_
if test $? -ne 0 ; then fail 1 ; fi
subdir/d >d.1.out
if test $? -ne 0 ; then fail 1 ; fi
diff - d.1.out <<'_EOF_'
subdir/d.c
_EOF_
if test $? -ne 0 ; then fail 1 ; fi

#
pass
